@@ -10,11 +10,19 @@
 #ifndef slic3r_FillPlanePath_hpp_
 #define slic3r_FillPlanePath_hpp_
 
+#include <math.h>
+#include <stddef.h>
 #include <map>
+#include <algorithm>
+#include <utility>
+#include <cmath>
+#include <cstddef>
 
-#include "../libslic3r.h"
-
+#include "libslic3r/libslic3r.h"
 #include "FillBase.hpp"
+#include "libslic3r/ExPolygon.hpp"
+#include "libslic3r/Point.hpp"
+#include "libslic3r/Polyline.hpp"
 
 namespace Slic3r {
 
