@@ -5,9 +5,16 @@
 #ifndef slic3r_AvoidCrossingPerimeters_hpp_
 #define slic3r_AvoidCrossingPerimeters_hpp_
 
-#include "../libslic3r.h"
-#include "../ExPolygon.hpp"
-#include "../EdgeGrid.hpp"
+#include <vector>
+
+#include "libslic3r/libslic3r.h"
+#include "libslic3r/ExPolygon.hpp"
+#include "libslic3r/EdgeGrid.hpp"
+#include "libslic3r/BoundingBox.hpp"
+#include "libslic3r/Layer.hpp"
+#include "libslic3r/Polygon.hpp"
+#include "libslic3r/Polyline.hpp"
+#include "libslic3r/ShortestPath.hpp"
 
 namespace Slic3r {
 
