@@ -5,22 +5,7 @@
 #ifndef slic3r_ExtrusionProcessor_hpp_
 #define slic3r_ExtrusionProcessor_hpp_
 
-#include "../AABBTreeLines.hpp"
-#include "../SupportSpotsGenerator.hpp"
-#include "../libslic3r.h"
-#include "../ExtrusionEntity.hpp"
-#include "../Layer.hpp"
-#include "../Point.hpp"
-#include "../SVG.hpp"
-#include "../BoundingBox.hpp"
-#include "../Polygon.hpp"
-#include "../ClipperUtils.hpp"
-#include "../Flow.hpp"
-#include "../Config.hpp"
-#include "../Line.hpp"
-#include "../Exception.hpp"
-#include "../PrintConfig.hpp"
-
+#include <stdlib.h>
 #include <algorithm>
 #include <cassert>
 #include <cmath>
@@ -33,8 +18,36 @@
 #include <unordered_map>
 #include <utility>
 #include <vector>
+#include <functional>
+#include <cstdlib>
 
-namespace Slic3r { namespace ExtrusionProcessor {
+#include "libslic3r/AABBTreeLines.hpp"
+#include "libslic3r/SupportSpotsGenerator.hpp"
+#include "libslic3r/libslic3r.h"
+#include "libslic3r/ExtrusionEntity.hpp"
+#include "libslic3r/Layer.hpp"
+#include "libslic3r/Point.hpp"
+#include "libslic3r/SVG.hpp"
+#include "libslic3r/BoundingBox.hpp"
+#include "libslic3r/Polygon.hpp"
+#include "libslic3r/ClipperUtils.hpp"
+#include "libslic3r/Flow.hpp"
+#include "libslic3r/Config.hpp"
+#include "libslic3r/Line.hpp"
+#include "libslic3r/Exception.hpp"
+#include "libslic3r/PrintConfig.hpp"
+#include "libslic3r/ExtrusionEntityCollection.hpp"
+
+namespace Slic3r {
+class CurledLine;
+class Linef;
+}  // namespace Slic3r
+
+namespace Slic3r::ExtrusionProcessor {
+
+// Minimum decrease of the fan speed in percents that will be emitted into g-code.
+// Decreases below this limit will be omitted to not overflow the g-code with fan speed changes.
+const constexpr float MIN_FAN_SPEED_NEGATIVE_CHANGE_TO_EMIT = 3.f;
 
 struct ExtendedPoint
 {
@@ -43,13 +56,26 @@ struct ExtendedPoint
     float curvature;
 };
 
-template<bool SCALED_INPUT, bool ADD_INTERSECTIONS, bool PREV_LAYER_BOUNDARY_OFFSET, bool SIGNED_DISTANCE, typename POINTS, typename L>
-std::vector<ExtendedPoint> estimate_points_properties(const POINTS                           &input_points,
-                                                      const AABBTreeLines::LinesDistancer<L> &unscaled_prev_layer,
-                                                      float                                   flow_width,
-                                                      float                                   max_line_length = -1.0f)
+struct OverhangSpeeds
 {
-    bool   looped     = input_points.front() == input_points.back();
+    float print_speed;
+    float fan_speed;
+};
+
+struct PropertiesEstimationConfig {
+    bool add_corners{};
+    bool prev_layer_boundary_offset{};
+    float flow_width;
+    float max_line_length{-1.0f};
+};
+
+template<bool SIGNED_DISTANCE, typename POINTS, typename L>
+std::vector<ExtendedPoint> estimate_points_properties(
+    const POINTS &input_points,
+    const AABBTreeLines::LinesDistancer<L> &unscaled_prev_layer,
+    const PropertiesEstimationConfig &config
+) {
+    bool looped = input_points.front() == input_points.back();
     std::function<size_t(size_t,size_t)> get_prev_index = [](size_t idx, size_t count) {
         if (idx > 0) {
             return idx - 1;
@@ -77,32 +103,28 @@ std::vector<ExtendedPoint> estimate_points_properties(const POINTS
         };
     };
 
-    using P = typename POINTS::value_type;
-
     using AABBScalar = typename AABBTreeLines::LinesDistancer<L>::Scalar;
     if (input_points.empty())
         return {};
-    float boundary_offset = PREV_LAYER_BOUNDARY_OFFSET ? 0.5 * flow_width : 0.0f;
-    auto  maybe_unscale   = [](const P &p) { return SCALED_INPUT ? unscaled(p) : p.template cast<double>(); };
+    float boundary_offset = config.prev_layer_boundary_offset ? 0.5 * config.flow_width : 0.0f;
 
     std::vector<ExtendedPoint> points;
-    points.reserve(input_points.size() * (ADD_INTERSECTIONS ? 1.5 : 1));
+    points.reserve(input_points.size() * 1.5);
 
     {
-        ExtendedPoint start_point{maybe_unscale(input_points.front())};
+        ExtendedPoint start_point{unscaled(input_points.front())};
         auto [distance, nearest_line,
               x] = unscaled_prev_layer.template distance_from_lines_extra<SIGNED_DISTANCE>(start_point.position.cast<AABBScalar>());
         start_point.distance = distance + boundary_offset;
         points.push_back(start_point);
     }
     for (size_t i = 1; i < input_points.size(); i++) {
-        ExtendedPoint next_point{maybe_unscale(input_points[i])};
+        ExtendedPoint next_point{unscaled(input_points[i])};
         auto [distance, nearest_line,
               x] = unscaled_prev_layer.template distance_from_lines_extra<SIGNED_DISTANCE>(next_point.position.cast<AABBScalar>());
         next_point.distance = distance + boundary_offset;
 
-        if (ADD_INTERSECTIONS &&
-            ((points.back().distance > boundary_offset + EPSILON) != (next_point.distance > boundary_offset + EPSILON))) {
+        if (((points.back().distance > boundary_offset + EPSILON) != (next_point.distance > boundary_offset + EPSILON))) {
             const ExtendedPoint &prev_point    = points.back();
             auto                 intersections = unscaled_prev_layer.template intersections_with_line<true>(
                 L{prev_point.position.cast<AABBScalar>(), next_point.position.cast<AABBScalar>()});
@@ -116,7 +138,7 @@ std::vector<ExtendedPoint> estimate_points_properties(const POINTS
         points.push_back(next_point);
     }
 
-    if (PREV_LAYER_BOUNDARY_OFFSET && ADD_INTERSECTIONS) {
+    if (config.add_corners) {
         std::vector<ExtendedPoint> new_points;
         new_points.reserve(points.size() * 2);
         new_points.push_back(points.front());
@@ -158,7 +180,7 @@ std::vector<ExtendedPoint> estimate_points_properties(const POINTS
         points = std::move(new_points);
     }
 
-    if (max_line_length > 0) {
+    if (config.max_line_length > 0) {
         std::vector<ExtendedPoint> new_points;
         new_points.reserve(points.size() * 2);
         {
@@ -167,7 +189,7 @@ std::vector<ExtendedPoint> estimate_points_properties(const POINTS
                 const ExtendedPoint &next = points[i + 1];
                 new_points.push_back(curr);
                 double len             = (next.position - curr.position).squaredNorm();
-                double t               = sqrt((max_line_length * max_line_length) / len);
+                double t               = sqrt((config.max_line_length * config.max_line_length) / len);
                 size_t new_point_count = 1.0 / t;
                 for (size_t j = 1; j < new_point_count + 1; j++) {
                     Vec2d pos  = curr.position * (1.0 - j * t) + next.position * (j * t);
@@ -258,12 +280,13 @@ ExtrusionEntityCollection calculate_and_split_overhanging_extrusions(
     const AABBTreeLines::LinesDistancer<Linef>      &unscaled_prev_layer,
     const AABBTreeLines::LinesDistancer<CurledLine> &prev_layer_curled_lines);
 
-std::pair<float, float> calculate_overhang_speed(const ExtrusionAttributes &attributes,
-                                                 const FullPrintConfig     &config,
-                                                 size_t                     extruder_id,
-                                                 float                      external_perim_reference_speed,
-                                                 float                      default_speed);
+OverhangSpeeds calculate_overhang_speed(const ExtrusionAttributes  &attributes,
+                                        const FullPrintConfig      &config,
+                                        size_t                      extruder_id,
+                                        float                       external_perimeter_reference_speed,
+                                        float                       default_speed,
+                                        const std::optional<float> &current_fan_speed);
 
-}} // namespace Slic3r::ExtrusionProcessor
+} // namespace Slic3r::ExtrusionProcessor
 
 #endif // slic3r_ExtrusionProcessor_hpp_
