@@ -74,27 +74,46 @@ public:
     std::string toolchange(unsigned int extruder_id);
     std::string set_speed(double F, const std::string_view comment = {}, const std::string_view cooling_marker = {}) const;
 
-    std::string get_travel_to_xy_gcode(const Vec2d &point, const std::string_view comment) const;
-    std::string travel_to_xy(const Vec2d &point, const std::string_view comment = {});
-    std::string travel_to_xy_G2G3IJ(const Vec2d &point, const Vec2d &ij, const bool ccw, const std::string_view comment = {});
-
     /**
-     * @brief Return gcode with all three axis defined. Optionally adds feedrate.
+     * @brief Return gcode to travel to the specified point.
+     * Feed rate is computed based on the vector (to - m_pos).
+     * Maintains the internal m_pos position.
+     * Movements less than XYZ_EPSILON generate no output.
      *
-     * Feedrate is added the starting point "from" is specified.
-     *
-     * @param from Optional starting point of the travel.
      * @param to Where to travel to.
      * @param comment Description of the travel purpose.
      */
-    std::string get_travel_to_xyz_gcode(const Vec3d &to, const std::string_view comment) const;
     std::string travel_to_xyz(const Vec3d &to, const std::string_view comment = {});
-    std::string get_travel_to_z_gcode(double z, const std::string_view comment) const;
+    std::string travel_to_xy(const Vec2d &point, const std::string_view comment = {});
     std::string travel_to_z(double z, const std::string_view comment = {});
+
+    std::string travel_to_xy_G2G3IJ(const Vec2d &point, const Vec2d &ij, const bool ccw, const std::string_view comment = {});
+
+    /**
+     * @brief Generate G-Code to travel to the specified point unconditionally.
+     * Feed rate is computed based on the vector (to - m_pos).
+     * Maintains the internal m_pos position.
+     * The distance test XYZ_EPSILON is not performed.
+     * @param to The point to travel to.
+     * @param comment Description of the travel purpose.
+     */
+    std::string travel_to_xyz_force(const Vec3d &to, const std::string_view comment = {});
+    std::string travel_to_xy_force(const Vec2d &point, const std::string_view comment = {});
+    std::string travel_to_z_force(double z, const std::string_view comment = {});
+
+    /**
+     * @brief Generate G-Code to move to the specified point while extruding.
+     * Maintains the internal m_pos position.
+     * The distance test XYZ_EPSILON is not performed.
+     * @param point The point to move to.
+     * @param dE The E-steps to extrude while moving.
+     * @param comment Description of the movement purpose.
+     */
     std::string extrude_to_xy(const Vec2d &point, double dE, const std::string_view comment = {});
     std::string extrude_to_xyz(const Vec3d &point, double dE, const std::string_view comment = {});
+
     std::string extrude_to_xy_G2G3IJ(const Vec2d &point, const Vec2d &ij, const bool ccw, double dE, const std::string_view comment);
-//    std::string extrude_to_xyz(const Vec3d &point, double dE, const std::string_view comment = {});
+
     std::string retract(bool before_wipe = false);
     std::string retract_for_toolchange(bool before_wipe = false);
     std::string unretract();
@@ -176,6 +195,9 @@ public:
     static constexpr const std::array<double, 10> pow_10    {   1.,     10.,    100.,    1000.,    10000.,    100000.,    1000000.,    10000000.,    100000000.,    1000000000.};
     static constexpr const std::array<double, 10> pow_10_inv{1./1.,  1./10., 1./100., 1./1000., 1./10000., 1./100000., 1./1000000., 1./10000000., 1./100000000., 1./1000000000.};
 
+    // Compute XYZ_EPSILON based on XYZF_EXPORT_DIGITS
+    static constexpr double XYZ_EPSILON = pow_10_inv[XYZF_EXPORT_DIGITS];
+
     // Quantize doubles to a resolution of the G-code.
     static double                                 quantize(double v, size_t ndigits) { return std::round(v * pow_10[ndigits]) * pow_10_inv[ndigits]; }
     static double                                 quantize_xyzf(double v) { return quantize(v, XYZF_EXPORT_DIGITS); }
