@@ -15,21 +15,13 @@
 #include "libslic3r.h"
 #include "BuildVolume.hpp"
 #include "Exception.hpp"
-#include "Model.hpp"
+
 #include "Geometry/ConvexHull.hpp"
 #include "MTUtils.hpp"
 #include "TriangleMeshSlicer.hpp"
 #include "TriangleSelector.hpp"
 #include "MultipleBeds.hpp"
 
-#include "Format/AMF.hpp"
-#include "Format/OBJ.hpp"
-#include "Format/STL.hpp"
-#include "Format/3mf.hpp"
-#include "Format/STEP.hpp"
-#include "Format/SVG.hpp"
-#include "Format/PrintRequest.hpp"
-
 #include <float.h>
 
 #include <boost/algorithm/string/predicate.hpp>
@@ -148,109 +140,6 @@ const CustomGCode::Info& Model::custom_gcode_per_print_z() const
 {
     return custom_gcode_per_print_z_vector[s_multiple_beds.get_active_bed()];
 }
-
-
-// Loading model from a file, it may be a simple geometry file as STL or OBJ, however it may be a project file as well.
-Model Model::read_from_file(const std::string& input_file, DynamicPrintConfig* config, ConfigSubstitutionContext* config_substitutions, LoadAttributes options)
-{
-    Model model;
-
-    DynamicPrintConfig temp_config;
-    ConfigSubstitutionContext temp_config_substitutions_context(ForwardCompatibilitySubstitutionRule::EnableSilent);
-    if (config == nullptr)
-        config = &temp_config;
-    if (config_substitutions == nullptr)
-        config_substitutions = &temp_config_substitutions_context;
-
-    bool result = false;
-    if (boost::algorithm::iends_with(input_file, ".stl"))
-        result = load_stl(input_file.c_str(), &model);
-    else if (boost::algorithm::iends_with(input_file, ".obj"))
-        result = load_obj(input_file.c_str(), &model);
-    else if (boost::algorithm::iends_with(input_file, ".step") || boost::algorithm::iends_with(input_file, ".stp"))
-        result = load_step(input_file.c_str(), &model);
-    else if (boost::algorithm::iends_with(input_file, ".amf") || boost::algorithm::iends_with(input_file, ".amf.xml"))
-        result = load_amf(input_file.c_str(), config, config_substitutions, &model, options & LoadAttribute::CheckVersion);
-    else if (boost::algorithm::iends_with(input_file, ".3mf") || boost::algorithm::iends_with(input_file, ".zip")) {
-        //FIXME options & LoadAttribute::CheckVersion ? 
-        boost::optional<Semver> prusaslicer_generator_version;
-        result = load_3mf(input_file.c_str(), *config, *config_substitutions, &model, false, prusaslicer_generator_version);
-    } else if (boost::algorithm::iends_with(input_file, ".svg"))
-        result = load_svg(input_file, model);
-    else if (boost::ends_with(input_file, ".printRequest"))
-        result = load_printRequest(input_file.c_str(), &model);
-    else
-        throw Slic3r::RuntimeError("Unknown file format. Input file must have .stl, .obj, .step/.stp, .svg, .amf(.xml) or extension .3mf(.zip).");
-
-    if (! result)
-        throw Slic3r::RuntimeError("Loading of a model file failed.");
-
-    if (model.objects.empty())
-        throw Slic3r::RuntimeError("The supplied file couldn't be read because it's empty");
-
-    if (!boost::ends_with(input_file, ".printRequest"))
-        for (ModelObject *o : model.objects)
-            o->input_file = input_file;
-
-    if (options & LoadAttribute::AddDefaultInstances)
-        model.add_default_instances();
-
-    for (CustomGCode::Info& info : model.custom_gcode_per_print_z_vector) {
-        CustomGCode::update_custom_gcode_per_print_z_from_config(info, config);
-        CustomGCode::check_mode_for_custom_gcode_per_print_z(info);
-    }
-
-    sort_remove_duplicates(config_substitutions->substitutions);
-    return model;
-}
-
-// Loading model from a file (3MF or AMF), not from a simple geometry file (STL or OBJ).
-Model Model::read_from_archive(
-    const std::string& input_file,
-    DynamicPrintConfig* config,
-    ConfigSubstitutionContext* config_substitutions,
-    boost::optional<Semver> &prusaslicer_generator_version,
-    LoadAttributes options
-) {
-    assert(config != nullptr);
-    assert(config_substitutions != nullptr);
-
-    Model model;
-
-    bool result = false;
-    if (boost::algorithm::iends_with(input_file, ".3mf") || boost::algorithm::iends_with(input_file, ".zip")) {
-        result = load_3mf(input_file.c_str(), *config, *config_substitutions, &model, options & LoadAttribute::CheckVersion, prusaslicer_generator_version);
-    } else if (boost::algorithm::iends_with(input_file, ".zip.amf"))
-        result = load_amf(input_file.c_str(), config, config_substitutions, &model, options & LoadAttribute::CheckVersion);
-    else
-        throw Slic3r::RuntimeError("Unknown file format. Input file must have .3mf or .zip.amf extension.");
-
-    if (!result)
-        throw Slic3r::RuntimeError("Loading of a model file failed.");
-
-    for (ModelObject *o : model.objects) {
-//        if (boost::algorithm::iends_with(input_file, ".zip.amf"))
-//        {
-//            // we remove the .zip part of the extension to avoid it be added to filenames when exporting
-//            o->input_file = boost::ireplace_last_copy(input_file, ".zip.", ".");
-//        }
-//        else
-            o->input_file = input_file;
-    }
-
-    if (options & LoadAttribute::AddDefaultInstances)
-        model.add_default_instances();
-
-    for (CustomGCode::Info& info : model.custom_gcode_per_print_z_vector) {
-        CustomGCode::update_custom_gcode_per_print_z_from_config(info, config);
-        CustomGCode::check_mode_for_custom_gcode_per_print_z(info);
-    }
-
-    handle_legacy_sla(*config);
-
-    return model;
-}
-
 ModelObject* Model::add_object()
 {
     this->objects.emplace_back(new ModelObject(this));
@@ -472,169 +361,6 @@ void Model::duplicate_objects_grid(size_t x, size_t y, coordf_t dist)
     }
 }
 
-bool Model::looks_like_multipart_object() const
-{
-    if (this->objects.size() <= 1)
-        return false;
-
-    BoundingBoxf3 tbb;
-
-    for (const ModelObject *obj : this->objects) {
-        if (obj->volumes.size() > 1 || obj->config.keys().size() > 1)
-            return false;
-
-        BoundingBoxf3 bb_this = obj->volumes[0]->mesh().bounding_box();
-
-        // FIXME: There is sadly the case when instances are empty (AMF files). The normalization of instances in that
-        // case is performed only after this function is called. For now (shortly before the 2.7.2 release, let's
-        // just do this non-invasive check. Reordering all the functions could break it much more.
-        BoundingBoxf3 tbb_this = (! obj->instances.empty() ? obj->instances[0]->transform_bounding_box(bb_this) : bb_this);
-
-        if (!tbb.defined)
-            tbb = tbb_this;
-        else if (tbb.intersects(tbb_this) || tbb.shares_boundary(tbb_this))
-            // The volumes has intersects bounding boxes or share some boundary
-            return true;
-    }
-    return false;
-}
-
-// Generate next extruder ID string, in the range of (1, max_extruders).
-static inline int auto_extruder_id(unsigned int max_extruders, unsigned int &cntr)
-{
-    int out = ++ cntr;
-    if (cntr == max_extruders)
-    	cntr = 0;
-    return out;
-}
-
-void Model::convert_multipart_object(unsigned int max_extruders)
-{
-	assert(this->objects.size() >= 2);
-    if (this->objects.size() < 2)
-        return;
-    
-    ModelObject* object = new ModelObject(this);
-    object->input_file = this->objects.front()->input_file;
-    object->name = boost::filesystem::path(this->objects.front()->input_file).stem().string();
-    //FIXME copy the config etc?
-
-    unsigned int extruder_counter = 0;
-	for (const ModelObject* o : this->objects)
-    	for (const ModelVolume* v : o->volumes) {
-            // If there are more than one object, put all volumes together 
-            // Each object may contain any number of volumes and instances
-            // The volumes transformations are relative to the object containing them...
-            Geometry::Transformation trafo_volume = v->get_transformation();
-            // Revert the centering operation.
-            trafo_volume.set_offset(trafo_volume.get_offset() - o->origin_translation);
-            int counter = 1;
-            auto copy_volume = [o, max_extruders, &counter, &extruder_counter](ModelVolume *new_v) {
-                assert(new_v != nullptr);
-                new_v->name = (counter > 1) ? o->name + "_" + std::to_string(counter++) : o->name;
-                new_v->config.set("extruder", auto_extruder_id(max_extruders, extruder_counter));
-                return new_v;
-            };
-            if (o->instances.empty()) {
-            	copy_volume(object->add_volume(*v))->set_transformation(trafo_volume);
-            } else {
-            	for (const ModelInstance* i : o->instances)
-                    // ...so, transform everything to a common reference system (world)
-                	copy_volume(object->add_volume(*v))->set_transformation(i->get_transformation() * trafo_volume);                    
-            }
-        }
-
-    // commented-out to fix #2868
-//    object->add_instance();
-//    object->instances[0]->set_offset(object->raw_mesh_bounding_box().center());
-
-    this->clear_objects();
-    this->objects.push_back(object);
-}
-
-static constexpr const double volume_threshold_inches = 9.0; // 9 = 3*3*3;
-
-bool Model::looks_like_imperial_units() const
-{
-    if (this->objects.empty())
-        return false;
-
-    for (ModelObject* obj : this->objects)
-        if (obj->get_object_stl_stats().volume < volume_threshold_inches) {
-            if (!obj->is_cut())
-                return true;
-            bool all_cut_parts_look_like_imperial_units = true;
-            for (ModelObject* obj_other : this->objects) {
-                if (obj_other == obj)
-                    continue;
-                if (obj_other->cut_id.is_equal(obj->cut_id) && obj_other->get_object_stl_stats().volume >= volume_threshold_inches) {
-                    all_cut_parts_look_like_imperial_units = false;
-                    break;
-                }
-            }
-            if (all_cut_parts_look_like_imperial_units)
-                return true;
-        }
-
-    return false;
-}
-
-void Model::convert_from_imperial_units(bool only_small_volumes)
-{
-    static constexpr const float in_to_mm = 25.4f;
-    for (ModelObject* obj : this->objects)
-        if (! only_small_volumes || obj->get_object_stl_stats().volume < volume_threshold_inches) {
-            obj->scale_mesh_after_creation(in_to_mm);
-            for (ModelVolume* v : obj->volumes) {
-                assert(! v->source.is_converted_from_meters);
-                v->source.is_converted_from_inches = true;
-            }
-        }
-}
-
-static constexpr const double volume_threshold_meters = 0.001; // 0.001 = 0.1*0.1*0.1
-
-bool Model::looks_like_saved_in_meters() const
-{
-    if (this->objects.size() == 0)
-        return false;
-
-    for (ModelObject* obj : this->objects)
-        if (obj->get_object_stl_stats().volume < volume_threshold_meters)
-            return true;
-
-    return false;
-}
-
-void Model::convert_from_meters(bool only_small_volumes)
-{
-    static constexpr const double m_to_mm = 1000;
-    for (ModelObject* obj : this->objects)
-        if (! only_small_volumes || obj->get_object_stl_stats().volume < volume_threshold_meters) {
-            obj->scale_mesh_after_creation(m_to_mm);
-            for (ModelVolume* v : obj->volumes) {
-                assert(! v->source.is_converted_from_inches);
-                v->source.is_converted_from_meters = true;
-            }
-        }
-}
-
-static constexpr const double zero_volume = 0.0000000001;
-
-int Model::removed_objects_with_zero_volume()
-{
-    if (objects.size() == 0)
-        return 0;
-
-    int removed = 0;
-    for (int i = int(objects.size()) - 1; i >= 0; i--)
-        if (objects[i]->get_object_stl_stats().volume < zero_volume) {
-            delete_object(size_t(i));
-            removed++;
-        }
-    return removed;
-}
-
 void Model::adjust_min_z()
 {
     if (objects.empty())
@@ -833,6 +559,13 @@ ModelVolume* ModelObject::add_volume(const ModelVolume &other, TriangleMesh &&me
     return v;
 }
 
+ModelVolume* ModelObject::insert_volume(size_t idx, const ModelVolume& other, TriangleMesh&& mesh)
+{
+    ModelVolume* v = new ModelVolume(this, other, std::move(mesh));
+    this->volumes.insert(this->volumes.begin() + idx, v);
+    return v;
+}
+
 void ModelObject::delete_volume(size_t idx)
 {
     ModelVolumePtrs::iterator i = this->volumes.begin() + idx;
@@ -1264,72 +997,6 @@ void ModelObject::scale_mesh_after_creation(const float scale)
     this->invalidate_bounding_box();
 }
 
-void ModelObject::convert_units(ModelObjectPtrs& new_objects, ConversionType conv_type, std::vector<int> volume_idxs)
-{
-    BOOST_LOG_TRIVIAL(trace) << "ModelObject::convert_units - start";
-
-    ModelObject* new_object = new_clone(*this);
-
-    float koef = conv_type == ConversionType::CONV_FROM_INCH   ? 25.4f  : conv_type == ConversionType::CONV_TO_INCH  ? 0.0393700787f  :
-                 conv_type == ConversionType::CONV_FROM_METER  ? 1000.f : conv_type == ConversionType::CONV_TO_METER ? 0.001f         : 1.f;
-
-    new_object->set_model(nullptr);
-    new_object->sla_support_points.clear();
-    new_object->sla_drain_holes.clear();
-    new_object->sla_points_status = sla::PointsStatus::NoPoints;
-    new_object->clear_volumes();
-    new_object->input_file.clear();
-
-    int vol_idx = 0;
-    for (ModelVolume* volume : volumes) {
-        if (!volume->mesh().empty()) {
-            TriangleMesh mesh(volume->mesh());
-
-            ModelVolume* vol = new_object->add_volume(mesh);
-            vol->name = volume->name;
-            vol->set_type(volume->type());
-            // Don't copy the config's ID.
-            vol->config.assign_config(volume->config);
-            assert(vol->config.id().valid());
-            assert(vol->config.id() != volume->config.id());
-            vol->set_material(volume->material_id(), *volume->material());
-            vol->source.input_file = volume->source.input_file;
-            vol->source.object_idx = (int)new_objects.size();
-            vol->source.volume_idx = vol_idx;
-            vol->source.is_converted_from_inches = volume->source.is_converted_from_inches;
-            vol->source.is_converted_from_meters = volume->source.is_converted_from_meters;
-            vol->source.is_from_builtin_objects = volume->source.is_from_builtin_objects;
-
-            vol->supported_facets.assign(volume->supported_facets);
-            vol->seam_facets.assign(volume->seam_facets);
-            vol->mm_segmentation_facets.assign(volume->mm_segmentation_facets);
-            vol->fuzzy_skin_facets.assign(volume->fuzzy_skin_facets);
-
-            // Perform conversion only if the target "imperial" state is different from the current one.
-            // This check supports conversion of "mixed" set of volumes, each with different "imperial" state.
-            if (//vol->source.is_converted_from_inches != from_imperial && 
-                (volume_idxs.empty() || 
-                 std::find(volume_idxs.begin(), volume_idxs.end(), vol_idx) != volume_idxs.end())) {
-                vol->scale_geometry_after_creation(koef);
-                vol->set_offset(Vec3d(koef, koef, koef).cwiseProduct(volume->get_offset()));
-                if (conv_type == ConversionType::CONV_FROM_INCH || conv_type == ConversionType::CONV_TO_INCH)
-                    vol->source.is_converted_from_inches = conv_type == ConversionType::CONV_FROM_INCH;
-                if (conv_type == ConversionType::CONV_FROM_METER || conv_type == ConversionType::CONV_TO_METER)
-                    vol->source.is_converted_from_meters = conv_type == ConversionType::CONV_FROM_METER;
-                assert(! vol->source.is_converted_from_inches || ! vol->source.is_converted_from_meters);
-            }
-            else
-                vol->set_offset(volume->get_offset());
-        }
-        vol_idx ++;
-    }
-    new_object->invalidate_bounding_box();
-
-    new_objects.push_back(new_object);
-
-    BOOST_LOG_TRIVIAL(trace) << "ModelObject::convert_units - end";
-}
-
 size_t ModelObject::materials_count() const
 {
     std::set<t_model_material_id> material_ids;
@@ -1420,111 +1087,6 @@ void ModelVolume::reset_extra_facets()
 }
 
 
-/// <summary>
-/// Compare TriangleMeshes by Bounding boxes (mainly for sort)
-/// From Front(Z) Upper(Y) TopLeft(X) corner.
-/// 1. Seraparate group not overlaped i Z axis
-/// 2. Seraparate group not overlaped i Y axis
-/// 3. Start earlier in X (More on left side)
-/// </summary>
-/// <param name="triangle_mesh1">Compare from</param>
-/// <param name="triangle_mesh2">Compare to</param>
-/// <returns>True when triangle mesh 1 is closer, upper or lefter than triangle mesh 2 other wise false</returns>
-static bool is_front_up_left(const TriangleMesh &trinagle_mesh1, const TriangleMesh &triangle_mesh2)
-{
-    // stats form t1
-    const Vec3f &min1 = trinagle_mesh1.stats().min;
-    const Vec3f &max1 = trinagle_mesh1.stats().max;
-    // stats from t2
-    const Vec3f &min2 = triangle_mesh2.stats().min;
-    const Vec3f &max2 = triangle_mesh2.stats().max;
-    // priority Z, Y, X
-    for (int axe = 2; axe > 0; --axe) {
-        if (max1[axe] < min2[axe])
-            return true;
-        if (min1[axe] > max2[axe])
-            return false;
-    }
-    return min1.x() < min2.x();
-}
-
-void ModelObject::split(ModelObjectPtrs* new_objects)
-{
-    for (ModelVolume* volume : this->volumes) {
-        if (volume->type() != ModelVolumeType::MODEL_PART)
-            continue;
-
-        // splited volume should not be text object 
-        if (volume->text_configuration.has_value())
-            volume->text_configuration.reset();
-
-        std::vector<TriangleMesh> meshes = volume->mesh().split();
-        std::sort(meshes.begin(), meshes.end(), is_front_up_left);
-
-        size_t counter = 1;
-        for (TriangleMesh &mesh : meshes) {
-            // FIXME: crashes if not satisfied
-            if (mesh.facets_count() < 3 || mesh.has_zero_volume())
-                continue;
-
-            // XXX: this seems to be the only real usage of m_model, maybe refactor this so that it's not needed?
-            ModelObject* new_object = m_model->add_object();
-            if (meshes.size() == 1) {
-                new_object->name = volume->name;
-                // Don't copy the config's ID.
-                new_object->config.assign_config(this->config.size() > 0 ? this->config : volume->config);
-            }
-            else {
-                new_object->name = this->name + (meshes.size() > 1 ? "_" + std::to_string(counter++) : "");
-                // Don't copy the config's ID.
-                new_object->config.assign_config(this->config);
-            }
-            assert(new_object->config.id().valid());
-            assert(new_object->config.id() != this->config.id());
-            new_object->instances.reserve(this->instances.size());
-            for (const ModelInstance* model_instance : this->instances)
-                new_object->add_instance(*model_instance);
-            ModelVolume* new_vol = new_object->add_volume(*volume, std::move(mesh));
-
-            // Invalidate extruder value in volume's config,
-            // otherwise there will no way to change extruder for object after splitting,
-            // because volume's extruder value overrides object's extruder value.
-            if (new_vol->config.has("extruder"))
-                new_vol->config.set_key_value("extruder", new ConfigOptionInt(0));
-
-            for (ModelInstance* model_instance : new_object->instances) {
-                const Vec3d shift = model_instance->get_transformation().get_matrix_no_offset() * new_vol->get_offset();
-                model_instance->set_offset(model_instance->get_offset() + shift);
-            }
-
-            new_vol->set_offset(Vec3d::Zero());
-            // reset the source to disable reload from disk
-            new_vol->source = ModelVolume::Source();
-            new_objects->emplace_back(new_object);
-        }
-    }
-}
-
-
-void ModelObject::merge()
-{
-    if (this->volumes.size() == 1) {
-        // We can't merge meshes if there's just one volume
-        return;
-    }
-
-    TriangleMesh mesh;
-
-    for (ModelVolume* volume : volumes)
-        if (!volume->mesh().empty())
-            mesh.merge(volume->mesh());
-
-    this->clear_volumes();
-    ModelVolume* vol = this->add_volume(mesh);
-
-    if (!vol)
-        return;
-}
 
 // Support for non-uniform scaling of instances. If an instance is rotated by angles, which are not multiples of ninety degrees,
 // then the scaling in world coordinate system is not representable by the Geometry::Transformation structure.
@@ -1733,42 +1295,6 @@ std::string ModelObject::get_export_filename() const
     return ret;
 }
 
-TriangleMeshStats ModelObject::get_object_stl_stats() const
-{
-    TriangleMeshStats full_stats;
-    full_stats.volume = 0.f;
-
-    // fill full_stats from all objet's meshes
-    for (ModelVolume* volume : this->volumes)
-    {
-        const TriangleMeshStats& stats = volume->mesh().stats();
-
-        // initialize full_stats (for repaired errors)
-        full_stats.open_edges           += stats.open_edges;
-        full_stats.repaired_errors.merge(stats.repaired_errors);
-
-        // another used satistics value
-        if (volume->is_model_part()) {
-            Transform3d trans = instances.empty() ? volume->get_matrix() : (volume->get_matrix() * instances[0]->get_matrix());
-            full_stats.volume           += stats.volume * std::fabs(trans.matrix().block(0, 0, 3, 3).determinant());
-            full_stats.number_of_parts  += stats.number_of_parts;
-        }
-    }
-
-    return full_stats;
-}
-
-int ModelObject::get_repaired_errors_count(const int vol_idx /*= -1*/) const
-{
-    if (vol_idx >= 0)
-        return this->volumes[vol_idx]->get_repaired_errors_count();
-
-    const RepairedMeshErrors& stats = get_object_stl_stats().repaired_errors;
-
-    return  stats.degenerate_facets + stats.edges_fixed     + stats.facets_removed +
-            stats.facets_reversed + stats.backwards_edges;
-}
-
 bool ModelObject::has_solid_mesh() const
 {
     for (const ModelVolume* volume : volumes)
@@ -1849,14 +1375,6 @@ void ModelVolume::calculate_convex_hull()
     assert(m_convex_hull.get());
 }
 
-int ModelVolume::get_repaired_errors_count() const
-{
-    const RepairedMeshErrors &stats = this->mesh().stats().repaired_errors;
-
-    return  stats.degenerate_facets + stats.edges_fixed     + stats.facets_removed +
-            stats.facets_reversed + stats.backwards_edges;
-}
-
 const TriangleMesh& ModelVolume::get_convex_hull() const
 {
     return *m_convex_hull.get();
@@ -1897,68 +1415,6 @@ std::string ModelVolume::type_to_string(const ModelVolumeType t)
     }
 }
 
-// Split this volume, append the result to the object owning this volume.
-// Return the number of volumes created from this one.
-// This is useful to assign different materials to different volumes of an object.
-size_t ModelVolume::split(unsigned int max_extruders)
-{
-    std::vector<TriangleMesh> meshes = this->mesh().split();
-    if (meshes.size() <= 1)
-        return 1;
-
-    std::sort(meshes.begin(), meshes.end(), is_front_up_left);
-
-    // splited volume should not be text object
-    if (text_configuration.has_value())
-        text_configuration.reset();
-
-    size_t idx = 0;
-    size_t ivolume = std::find(this->object->volumes.begin(), this->object->volumes.end(), this) - this->object->volumes.begin();
-    const std::string& name = this->name;
-
-    unsigned int extruder_counter = 0;
-    const Vec3d offset = this->get_offset();
-
-    for (TriangleMesh &mesh : meshes) {
-        if (mesh.empty() || mesh.has_zero_volume())
-            // Repair may have removed unconnected triangles, thus emptying the mesh.
-            continue;
-
-        if (idx == 0) {
-            this->set_mesh(std::move(mesh));
-            this->calculate_convex_hull();
-            // Assign a new unique ID, so that a new GLVolume will be generated.
-            this->set_new_unique_id();
-            // reset the source to disable reload from disk
-            this->source = ModelVolume::Source();
-        }
-        else
-            this->object->volumes.insert(this->object->volumes.begin() + (++ivolume), new ModelVolume(object, *this, std::move(mesh)));
-
-        this->object->volumes[ivolume]->set_offset(Vec3d::Zero());
-        this->object->volumes[ivolume]->center_geometry_after_creation();
-        this->object->volumes[ivolume]->translate(offset);
-        this->object->volumes[ivolume]->name = name + "_" + std::to_string(idx + 1);
-        this->object->volumes[ivolume]->config.set("extruder", auto_extruder_id(max_extruders, extruder_counter));
-        this->object->volumes[ivolume]->m_is_splittable = 0;
-        ++ idx;
-    }
-
-    // discard volumes for which the convex hull was not generated or is degenerate
-    size_t i = 0;
-    while (i < this->object->volumes.size()) {
-        const std::shared_ptr<const TriangleMesh> &hull = this->object->volumes[i]->get_convex_hull_shared_ptr();
-        if (hull == nullptr || hull->its.vertices.empty() || hull->its.indices.empty()) {
-            this->object->delete_volume(i);
-            --idx;
-            --i;
-        }
-        ++i;
-    }
-
-    return idx;
-}
-
 void ModelVolume::translate(const Vec3d& displacement)
 {
     set_offset(get_offset() + displacement);
@@ -2052,22 +1508,6 @@ void ModelVolume::transform_this_mesh(const Matrix3d &matrix, bool fix_left_hand
     this->set_new_unique_id();
 }
 
-void ModelVolume::convert_from_imperial_units()
-{
-    assert(! this->source.is_converted_from_meters);
-    this->scale_geometry_after_creation(25.4f);
-    this->set_offset(Vec3d(0, 0, 0));
-    this->source.is_converted_from_inches = true;
-}
-
-void ModelVolume::convert_from_meters()
-{
-    assert(! this->source.is_converted_from_inches);
-    this->scale_geometry_after_creation(1000.f);
-    this->set_offset(Vec3d(0, 0, 0));
-    this->source.is_converted_from_meters = true;
-}
-
 std::vector<size_t> ModelVolume::get_extruders_from_multi_material_painting() const {
     if (!this->is_mm_painted())
         return {};
