@@ -6,12 +6,13 @@
 #include <string>
 #include <vector>
 
+struct stl_facet;
+
 namespace Slic3r {
 
 struct OCCTVolume {
-    std::string volume_name;
-    std::vector<std::array<float, 3>> vertices;
-    std::vector<std::array<int, 3>> indices;
+    std::string            volume_name;
+    std::vector<stl_facet> facets;
 };
 
 struct OCCTResult {
