@@ -34,6 +34,7 @@
 #include "libslic3r/ClipperUtils.hpp"
 #include "libslic3r/Tesselate.hpp"
 #include "libslic3r/PrintConfig.hpp"
+#include "libslic3r/MultipleBeds.hpp"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -248,7 +249,6 @@ GLVolume::GLVolume(float r, float g, float b, float a)
     , is_outside(false)
     , hover(HS_None)
     , is_modifier(false)
-    , is_wipe_tower(false)
     , is_extrusion_path(false)
     , force_native_color(false)
     , force_neutral_color(false)
@@ -500,13 +500,13 @@ int GLVolumeCollection::load_object_volume(
 }
 
 #if SLIC3R_OPENGL_ES
-int GLVolumeCollection::load_wipe_tower_preview(
+GLVolume* GLVolumeCollection::load_wipe_tower_preview(
     float pos_x, float pos_y, float width, float depth, const std::vector<std::pair<float, float>>& z_and_depth_pairs, float height, float cone_angle,
-    float rotation_angle, bool size_unknown, float brim_width, TriangleMesh* out_mesh)
+    float rotation_angle, bool size_unknown, float brim_width, size_t idx, TriangleMesh* out_mesh)
 #else
-int GLVolumeCollection::load_wipe_tower_preview(
+GLVolume* GLVolumeCollection::load_wipe_tower_preview(
     float pos_x, float pos_y, float width, float depth, const std::vector<std::pair<float, float>>& z_and_depth_pairs, float height, float cone_angle,
-    float rotation_angle, bool size_unknown, float brim_width)
+    float rotation_angle, bool size_unknown, float brim_width, size_t idx)
 #endif // SLIC3R_OPENGL_ES
 {
     if (height == 0.0f)
@@ -591,9 +591,8 @@ int GLVolumeCollection::load_wipe_tower_preview(
         mesh.merge(cone_mesh);
     }
 
-
-    volumes.emplace_back(new GLVolume(color));
-    GLVolume& v = *volumes.back();
+    GLVolume* result{new GLVolume(color)};
+    GLVolume& v = *result;
 #if SLIC3R_OPENGL_ES
     if (out_mesh != nullptr)
         *out_mesh = mesh;
@@ -604,12 +603,13 @@ int GLVolumeCollection::load_wipe_tower_preview(
     v.set_convex_hull(mesh.convex_hull_3d());
     v.set_volume_offset(Vec3d(pos_x, pos_y, 0.0));
     v.set_volume_rotation(Vec3d(0., 0., (M_PI / 180.) * rotation_angle));
-    v.composite_id = GLVolume::CompositeID(INT_MAX, 0, 0);
+    v.composite_id = GLVolume::CompositeID(INT_MAX - idx, 0, 0);
     v.geometry_id.first = 0;
-    v.geometry_id.second = wipe_tower_instance_id().id;
-    v.is_wipe_tower = true;
+    v.geometry_id.second = wipe_tower_instance_id(idx).id;
+    v.wipe_tower_bed_index = idx;
     v.shader_outside_printer_detection_enabled = !size_unknown;
-    return int(volumes.size() - 1);
+
+    return result;
 }
 
 // Load SLA auxiliary GLVolumes (for support trees or pad).
@@ -657,6 +657,7 @@ void GLVolumeCollection::load_object_auxiliary(
             std::shared_ptr<const indexed_triangle_set> preview_mesh_ptr = print_object->get_mesh_to_print();
             if (preview_mesh_ptr != nullptr)
                 backend_mesh = TriangleMesh(*preview_mesh_ptr);
+            backend_mesh.translate(s_multiple_beds.get_bed_translation(s_multiple_beds.get_active_bed()).cast<float>());
             if (!backend_mesh.empty()) {
                 backend_mesh.transform(mesh_trafo_inv);
                 TriangleMesh convex_hull = backend_mesh.convex_hull_3d();
@@ -671,6 +672,7 @@ void GLVolumeCollection::load_object_auxiliary(
     // Get the support mesh.
     if (milestone == SLAPrintObjectStep::slaposSupportTree) {
         TriangleMesh supports_mesh = print_object->support_mesh();
+        supports_mesh.translate(s_multiple_beds.get_bed_translation(s_multiple_beds.get_active_bed()).cast<float>());
         if (!supports_mesh.empty()) {
             supports_mesh.transform(mesh_trafo_inv);
             TriangleMesh convex_hull = supports_mesh.convex_hull_3d();
@@ -684,6 +686,7 @@ void GLVolumeCollection::load_object_auxiliary(
     // Get the pad mesh.
     if (milestone == SLAPrintObjectStep::slaposPad) {
         TriangleMesh pad_mesh = print_object->pad_mesh();
+        pad_mesh.translate(s_multiple_beds.get_bed_translation(s_multiple_beds.get_active_bed()).cast<float>());
         if (!pad_mesh.empty()) {
             pad_mesh.transform(mesh_trafo_inv);
             TriangleMesh convex_hull = pad_mesh.convex_hull_3d();
@@ -803,7 +806,7 @@ void GLVolumeCollection::render(GLVolumeCollection::ERenderType type, bool disab
         const int obj_idx = volume.first->object_idx();
         const int vol_idx = volume.first->volume_idx();
         const bool render_as_mmu_painted = is_render_as_mmu_painted_enabled && !volume.first->selected &&
-            !volume.first->is_outside && volume.first->hover == GLVolume::HS_None && !volume.first->is_wipe_tower && obj_idx >= 0 && vol_idx >= 0 &&
+            !volume.first->is_outside && volume.first->hover == GLVolume::HS_None && !volume.first->is_wipe_tower() && obj_idx >= 0 && vol_idx >= 0 &&
             !model_objects[obj_idx]->volumes[vol_idx]->mm_segmentation_facets.empty() &&
             type != GLVolumeCollection::ERenderType::Transparent; // to filter out shells (not very nice)
         volume.first->set_render_color(true);
@@ -882,7 +885,7 @@ void GLVolumeCollection::render(GLVolumeCollection::ERenderType type, bool disab
             shader->set_uniform("print_volume.xy_data", m_print_volume.data);
             shader->set_uniform("print_volume.z_data", m_print_volume.zs);
             shader->set_uniform("volume_world_matrix", world_matrix);
-            shader->set_uniform("slope.actived", m_slope.active && !volume.first->is_modifier && !volume.first->is_wipe_tower);
+            shader->set_uniform("slope.actived", m_slope.active && !volume.first->is_modifier && !volume.first->is_wipe_tower());
             shader->set_uniform("slope.volume_world_normal_matrix", static_cast<Matrix3f>(world_matrix_inv_transp.cast<float>()));
             shader->set_uniform("slope.normal_z", m_slope.normal_z);
 
@@ -1012,7 +1015,7 @@ void GLVolumeCollection::update_colors_by_extruder(const DynamicPrintConfig* con
     }
 
     for (GLVolume* volume : volumes) {
-        if (volume == nullptr || volume->is_modifier || volume->is_wipe_tower || volume->is_sla_pad() || volume->is_sla_support())
+        if (volume == nullptr || volume->is_modifier || volume->is_wipe_tower() || volume->is_sla_pad() || volume->is_sla_support())
             continue;
 
         int extruder_id = volume->extruder_id - 1;
