@@ -23,21 +23,6 @@ namespace GUI {
 const size_t DOWNLOAD_MAX_CHUNK_SIZE	= 10 * 1024 * 1024;
 const size_t DOWNLOAD_SIZE_LIMIT		= 1024 * 1024 * 1024;
 
-std::string FileGet::escape_url(const std::string& unescaped)
-{
-	std::string ret_val;
-	CURL* curl = curl_easy_init();
-	if (curl) {
-		int decodelen;
-		char* decoded = curl_easy_unescape(curl, unescaped.c_str(), unescaped.size(), &decodelen);
-		if (decoded) {
-			ret_val = std::string(decoded);
-			curl_free(decoded);
-		}
-		curl_easy_cleanup(curl);
-	}
-	return ret_val;
-}
 bool FileGet::is_subdomain(const std::string& url, const std::string& domain)
 {
 	// domain should be f.e. printables.com (.com including)
@@ -86,7 +71,7 @@ unsigned get_current_pid()
 }
 
 // int = DOWNLOAD ID; string = file path
-wxDEFINE_EVENT(EVT_DWNLDR_FILE_COMPLETE, wxCommandEvent);
+wxDEFINE_EVENT(EVT_DWNLDR_FILE_COMPLETE, Event<DownloadEventData>);
 // int = DOWNLOAD ID; string = error msg
 wxDEFINE_EVENT(EVT_DWNLDR_FILE_ERROR, wxCommandEvent);
 // int = DOWNLOAD ID; string = progress percent
@@ -112,17 +97,19 @@ struct FileGet::priv
 	std::atomic_bool m_stopped { false }; // either canceled or paused - download is not running
 	size_t m_written { 0 };
 	size_t m_absolute_size { 0 };
-	priv(int ID, std::string&& url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder);
+    bool m_load_after;
+	priv(int ID, std::string&& url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder, bool load_after);
 
 	void get_perform();
 };
 
-FileGet::priv::priv(int ID, std::string&& url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder)
+FileGet::priv::priv(int ID, std::string&& url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder, bool load_after)
 	: m_id(ID)
 	, m_url(std::move(url))
 	, m_filename(filename)
 	, m_evt_handler(evt_handler)
 	, m_dest_folder(dest_folder)
+    , m_load_after(load_after)
 {
 }
 
@@ -279,23 +266,8 @@ void FileGet::priv::get_perform()
 			m_evt_handler->QueueEvent(evt);
 		})
 		.on_complete([&](std::string body, unsigned /* http_status */) {
-
-			// TODO: perform a body size check
-			// 
-			//size_t body_size = body.size();
-			//if (body_size != expected_size) {
-			//	return;
-			//}
 			try
 			{
-				/*
-				if (m_written < body.size())
-				{
-					// this code should never be entered. As there should be on_progress call after last bit downloaded.
-					std::string part_for_write = body.substr(m_written);
-					fwrite(part_for_write.c_str(), 1, part_for_write.size(), file);
-				}
-				*/
 				fclose(file);
 				boost::filesystem::rename(m_tmp_path, dest_path);
 			}
@@ -309,18 +281,15 @@ void FileGet::priv::get_perform()
 				m_evt_handler->QueueEvent(evt);
 				return;
 			}
-
-			wxCommandEvent* evt = new wxCommandEvent(EVT_DWNLDR_FILE_COMPLETE);
-			evt->SetString(dest_path.wstring());
-			evt->SetInt(m_id);
-			m_evt_handler->QueueEvent(evt);
+            DownloadEventData event_data = {m_id, dest_path.wstring(), m_load_after};
+            wxQueueEvent(m_evt_handler, new Event<DownloadEventData>(EVT_DWNLDR_FILE_COMPLETE, event_data));
 		})
 		.perform_sync();
 
 }
 
-FileGet::FileGet(int ID, std::string url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder)
-	: p(new priv(ID, std::move(url), filename, evt_handler, dest_folder))
+FileGet::FileGet(int ID, std::string url, const std::string& filename, wxEvtHandler* evt_handler, const boost::filesystem::path& dest_folder, bool load_after)
+	: p(new priv(ID, std::move(url), filename, evt_handler, dest_folder, load_after))
 {}
 
 FileGet::FileGet(FileGet&& other) : p(std::move(other.p)) {}
