@@ -1,3 +1,7 @@
+///|/ Copyright (c) Prusa Research 2021 Tomáš Mészáros @tamasmeszaros
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifndef NOTIFICATIONPROGRESSINDICATOR_HPP
 #define NOTIFICATIONPROGRESSINDICATOR_HPP
 
@@ -9,7 +13,7 @@ class NotificationManager;
 
 class NotificationProgressIndicator: public ProgressIndicator {
     NotificationManager *m_nm = nullptr;
-
+    CancelFn m_cancelfn;
 public:
 
     explicit NotificationProgressIndicator(NotificationManager *nm);
