@@ -1,7 +1,9 @@
-#include <catch2/catch.hpp>
+#include <catch2/catch_test_macros.hpp>
+#include <catch2/catch_approx.hpp>
 #include <test_utils.hpp>
 
 #include <random>
+#include <algorithm>
 
 #include <libslic3r/ExtrusionEntity.hpp>
 #include <libslic3r/GCode/ExtrusionOrder.hpp>
@@ -12,6 +14,7 @@
 #include <libslic3r/libslic3r.h>
 
 using namespace Slic3r;
+using namespace Catch;
 
 TEST_CASE("arc basics", "[ArcWelder]") {
     using namespace Slic3r::Geometry;
