@@ -2,7 +2,7 @@
 #define NOMINMAX
 #endif
 
-#include <catch2/catch.hpp>
+#include <catch2/catch_test_macros.hpp>
 
 #include <numeric>
 
