@@ -5,11 +5,13 @@
  *  and cross product uses doubles
  */
 
-#include <catch2/catch.hpp>
+#include <catch2/catch_test_macros.hpp>
+#include <catch2/matchers/catch_matchers_all.hpp>
 #include <libslic3r/Point.hpp>
 #include "test_utils.hpp"
 
 using namespace Slic3r;
+using namespace Catch;
 
 TEST_CASE("Nearest point", "[Point]") {
     const Point point{10, 15};
@@ -29,6 +31,13 @@ TEST_CASE("Distance to line", "[Point]") {
     CHECK(line.distance_to(Point{50, 50}) == Approx(50));
     CHECK(line.perp_distance_to(Point{50, 50}) == Approx(50));
     CHECK(line.perp_distance_to(Point{150, 50}) == Approx(50));
+    
+    // possitive values are on the left side WRT line direction
+    CHECK(line.perp_signed_distance_to(Point{50, 50}) == Approx(50));
+    CHECK(line.perp_signed_distance_to(Point{50, -50}) == Approx(-50));
+    const Line line2{{0, 0}, {0, 100}};
+    CHECK(line2.perp_signed_distance_to(Point{50, 50}) == Approx(-50));
+    CHECK(line2.perp_signed_distance_to(Point{-50, 50}) == Approx(50));
 }
 
 TEST_CASE("Distance to diagonal line", "[Point]") {
